'use strict';

var browser = {}, Port, _;

if (document instanceof window.HTMLDocument) {
	browser = document.documentElement || document.createElementNS('http://www.w3.org/1999/xhtml', 'div');

	if (!window.requestAnimationFrame) {
		window.requestAnimationFrame = window.webkitRequestAnimationFrame
			|| window.mozRequestAnimationFrame || function(cb) { return window.setTimeout(cb, 20); };
	}

	if (browser.style) {
		_ = browser.style;

		browser = {
			'wheel': 'onwheel' in browser ? 'wheel' : 'mousewheel',
			'transform': ('webkitTransform' in _ ? 'webkitT' : 't') + 'ransform',
			'transform_css': ('webkitTransform' in _ ? '-webkit-' : '') + 'transform',
			'transition_css': ('webkitTransition' in _ ? '-webkit-' : '') + 'transition',
			'transitionend': 'webkitTransition' in _ ? 'webkitTransitionEnd' : 'transitionend',
			'box-sizing_css': ('MozBoxSizing' in _ ? '-moz-' : ('WebkitBoxSizing' in _ ? '-webkit-' : '')) + 'box-sizing',
			'zoom-in': this.chrome || this.mx || this.safari ? '-webkit-zoom-in' : 'zoom-in'
		};
		_ = null;
	}
}

function gen_nodes(host, nodes) {
	if (!host || !Array.isArray(nodes)) {
		return;
	}

	if (nodes.length) {
		var node, attr,
			i = 0,
			doc = host.ownerDocument,
			fragment = doc.createDocumentFragment();

		for (; i < nodes.length; ++i) {
			if (nodes[i]) {
				if (typeof nodes[i] === 'string') {
					node = document.createTextNode(nodes[i]);
				}
				else {
					node = document.createElement(nodes[i].tag);

					if (nodes[i].attrs) {
						for (attr in nodes[i].attrs) {
							// bypass CSP
							if (attr === 'style') {
								node.style.cssText = nodes[i].attrs[attr];
							}
							else {
								node.setAttribute(attr, nodes[i].attrs[attr]);
							}
						}
					}

					if (nodes[i].nodes) {
						gen_nodes(node, nodes[i].nodes);
					}
					else if (nodes[i].text) {
						node.textContent = nodes[i].text;
					}
				}

				fragment.appendChild(node);
			}
		}

		if (fragment.childNodes.length) {
			host.appendChild(fragment);
		}
	}

	return host;
}

function media_type(type) {
	if (type) {
		document.media_type = type;
	}

	if (document.media_type) {
		return document.media_type;
	}

	var d = document, img = d.images && d.images.length === 1 && d.images[0];

	if (img && img.parentNode === d.body && img.src === window.location.href) {
		return document.media_type = 'img';
	}

	d = d.querySelector('body > video:first-child, body > div:only-child > video:first-child');

	// Firefox doesn't handle redirects properly, so it gives the previous URL here
	if (d && (browser.firefox || d.currentSrc === window.location.href)) {
		return document.media_type = 'video';
	}
}

browser.opera = true;

var LNG;

Port = {
	listen: function(listener) {
		if (this.listener) {
			opera.extension.removeEventListener('message', this.listener, false);
		}

		this.listener = listener;

		if (listener) {
			opera.extension.addEventListener('message', this.listener, false);
		}
	},
	send: function(msg) {
		opera.extension.postMessage(msg);
	}
};

if (window.location.protocol === 'widget:') {
	_ = function(s) {
		return typeof LNG === 'object' && LNG[s] || s;
	};

	LNG = document.createElement('script');
	LNG.src = './' + location.pathname.slice(1, location.pathname.indexOf('.')) + '_lng.js';
	document.head.appendChild(LNG);

	browser.insertHTML = function(n, html) { n.innerHTML = html; };
}
